import 'package:flutter/material.dart';
import 'package:cached_network_image/cached_network_image.dart';
import 'package:flutter/painting.dart';
import 'package:vengamo_chat_ui/theme/app_color.dart';

import '../seenStatus.dart';

class ImageWidget extends StatefulWidget {

  const ImageWidget(
      {super.key,
      required this.imgUrl,
      required this.caption,
      required this.constraints,
      required this.color,
      required this.isMe,
      required this.message,
      required this.textColor,
      required this.ack,
      required this.time,
      required this.timeLabelColor});

  /// Represents the URL of an image.
  final String imgUrl;
  /// Represents the caption or textual description associated with an image.
  final String caption;

  /// Represents the constraints applied to a widget's size.
  ///
  /// The [constraints] parameter is used to define the maximum width and height that a widget can occupy.
  /// It is commonly used in layout calculations to determine the size and positioning of child widgets within a parent widget.
  ///
  /// Example usage:
  /// dart /// Container( /// constraints: BoxConstraints( /// maxWidth: constraints.maxWidth * 0.8, /// ), /// child: ... /// ) /// ///
  /// In this example, the maxWidth property of the constraints is set to 80% of the maximum width available to the parent widget.
  /// This constraint ensures that the child widget, wrapped inside the Container, does not exceed the specified maximum width.

  final BoxConstraints constraints;

  ///Background Color
  final Color color;
  /// this indicates if the message is from me or the other user
  final bool isMe;
  /// message content eg text 
  final String message;
  /// color that will be applied to the text
  final Color textColor;
  /// Message status example ( sent, delivered and seen)
  final Widget ack;
  /// message time
  final String time;
  /// color that will be applied on time label.
  final Color timeLabelColor;

  @override
  _ImageWidgetState createState() => _ImageWidgetState();
}

class _ImageWidgetState extends State<ImageWidget> {
  double? imageHeight;

  @override
  void initState() {
    super.initState();
    _getImageSize();
  }

  Future<void> _getImageSize() async {
    final imageProvider = CachedNetworkImageProvider(widget.imgUrl);
    final streamCompleter = imageProvider.resolve(const ImageConfiguration());
    final listener = ImageStreamListener((info, _) {
      final height = info.image.height.toDouble();
      setState(() {
        imageHeight = height;
      });
    });
    streamCompleter.addListener(listener);
  }

  @override
  void dispose() {
    final imageProvider = CachedNetworkImageProvider(widget.imgUrl);
    final streamCompleter = imageProvider.resolve(const ImageConfiguration());
    final listener = ImageStreamListener((_, __) {});
    streamCompleter.removeListener(listener);
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return Padding(
      padding: const EdgeInsets.symmetric(horizontal: 2),
      child: Container(
        decoration: BoxDecoration(
          color: widget.color,
          borderRadius: BorderRadius.all(Radius.circular(9)),
        ),
        height: 350,
        constraints: BoxConstraints(
          maxWidth: widget.constraints.maxWidth * 0.8,
        ),
        child: ClipRRect(
          borderRadius: BorderRadius.circular(9),
          child: Stack(
            children: [
              Padding(
                padding: const EdgeInsets.all(3),
                child: Container(
                  decoration: BoxDecoration(
                    borderRadius: BorderRadius.all(Radius.circular(9)),
                    image: DecorationImage(
                      image: CachedNetworkImageProvider(widget.imgUrl),
                      fit: BoxFit.cover,
                    ),
                  ),
                ),
              ),
               Positioned(
                bottom: 5,
                right: 6,
                child: ClipRRect(
                  borderRadius: BorderRadius.circular(18),
                  child: Container(
                      alignment: Alignment.center,
                      width: 80,
                      height: 25,
                      color: Colors.black.withOpacity(0.3),
                      child: SeenStatus(
                          isMe: widget.isMe,
                          time: widget.time,
                          dateTextColor: widget.timeLabelColor,
                          messageAck: widget.ack)
                     ),
                ),
              ),
            ],
          ),
        ),
      ),
    );
  }
}
